#!/usr/bin/env bash

function miner_ver() {
  local MINER_VER=$TEAMREDMINER_VER
  [[ -z $MINER_VER ]] && MINER_VER=$MINER_LATEST_VER
  echo "$MINER_VER"
}


function miner_config_echo() {
  local MINER_VER=
  MINER_VER=$(miner_ver)
  miner_echo_config_file "/hive/miners/$MINER_NAME/$MINER_VER/$MINER_NAME.conf"
}

function miner_config_gen() {
  local MINER_CONFIG="$MINER_DIR/$MINER_VER/$MINER_NAME.conf"
  mkfile_from_symlink "$MINER_CONFIG"

  [[ -z $TEAMREDMINER_ALGO ]] && TEAMREDMINER_ALGO=lyra2z

  local pools=
  pools=$(echo "$TEAMREDMINER_URL")
  if [[ $TEAMREDMINER_SORTPOOLS -eq 1 ]]; then
    echo "Sorting pools with stratum-ping"
    local sort_conf=
    sort_conf="-s -u $(echo "$TEAMREDMINER_TEMPLATE" | cut -d '.' -f 1 | cut -d '/' -f 1)"
    [[ -n $TEAMREDMINER_PASS ]] && conf+=" -p $TEAMREDMINER_PASS"
    pools=$(pool-test "$sort_conf" "$pools")
  fi

  local conf="-a ${TEAMREDMINER_ALGO}"
  local creds="-u ${TEAMREDMINER_TEMPLATE}"
  local pool=

  if [[ -n $TEAMREDMINER_WORKER && $TEAMREDMINER_ALGO == "ethash" ]]; then
    creds+=" --eth_worker $TEAMREDMINER_WORKER"
  elif [[ -n $TEAMREDMINER_WORKER && $TEAMREDMINER_ALGO == "etchash" ]]; then
    creds+=" --eth_worker $TEAMREDMINER_WORKER"
  elif [[ -n $TEAMREDMINER_WORKER && $TEAMREDMINER_ALGO == "nimiq" ]]; then
    creds+=" --nimiq_worker $TEAMREDMINER_WORKER"
  elif [[ -n $TEAMREDMINER_WORKER && $TEAMREDMINER_ALGO == cn* ]]; then
    creds+=" --rig_id $TEAMREDMINER_WORKER"
  elif [[ -n $TEAMREDMINER_WORKER ]]; then
    creds+=".$TEAMREDMINER_WORKER"
  fi

  [[ -n $TEAMREDMINER_PASS ]] && creds+=" -p ${TEAMREDMINER_PASS}"

  for pool in $pools; do
    if [[ $pool == *://* ]]; then
      # allow to use nicehash+tcp://, ethproxy+tcp://, stratum2+tcp:// or +ssl://
      [[ $pool =~ nicehash\+([^[:space:]]+) ]] && pool="stratum+${BASH_REMATCH[1]} --eth_stratum_mode=nicehash"
      [[ $pool =~ ethproxy\+([^[:space:]]+) ]] && pool="stratum+${BASH_REMATCH[1]} --eth_stratum_mode=ethproxy"
      [[ $pool =~ stratum2\+([^[:space:]]+) ]] && pool="stratum+${BASH_REMATCH[1]} --eth_stratum_mode=stratum"
    else
      if [[ $TEAMREDMINER_TLS -eq 1 ]]; then
        pool="stratum+ssl://${pool}"
      else
        pool="stratum+tcp://${pool}"
      fi
      [[ "$TEAMREDMINER_USER_CONFIG" =~ (--eth_stratum_mode=[^[:space:]]+) ]] && pool+=" ${BASH_REMATCH[1]}"
    fi

    conf+=" -o ${pool} ${creds}"
  done

  [[ -n $TEAMREDMINER_USER_CONFIG ]] && conf+=" ${TEAMREDMINER_USER_CONFIG}"

  [[ -z $TEAMREDMINER_ALGO2 ]] && echo "$conf" > "$MINER_CONFIG" && exit 0
  # dual algo params
  conf+=" --${TEAMREDMINER_ALGO2}"

  local creds2="-u ${TEAMREDMINER_TEMPLATE2}"
#  [[ -n $TEAMREDMINER_WORKER2 ]] && creds2+=".${TEAMREDMINER_WORKER2}"
  if [[ -n $TEAMREDMINER_WORKER2 && $TEAMREDMINER_ALGO2 == "zil" ]]; then
     creds2+=" --eth_worker $TEAMREDMINER_WORKER"
  else
     creds2+=".${TEAMREDMINER_WORKER2}"
  fi
  [[ -n $TEAMREDMINER_PASS2 ]] && creds2+=" -p ${TEAMREDMINER_PASS2}"

  pools=$(echo "$TEAMREDMINER_URL2")
  if [[ $TEAMREDMINER_SORTPOOLS -eq 1 ]]; then
    echo "Sorting pools with stratum-ping"
    sort_conf="-s -u $(echo "$TEAMREDMINER_TEMPLATE2" | cut -d '.' -f 1 | cut -d '/' -f 1)"
    [[ -n $TEAMREDMINER_PASS2 ]] && conf+=" -p $TEAMREDMINER_PASS2"
    pools=$(pool-test "$sort_conf" "$pools")
  fi

  for pool in $pools; do
    if [[ $pool != *://* ]]; then
      if [[ $TEAMREDMINER_TLS2 -eq 1 ]]; then
        pool="stratum+ssl://${pool}"
      else
        pool="stratum+tcp://${pool}"
      fi
    fi

    conf+=" -o $pool ${creds2}"
  done

  [[ -n $TEAMREDMINER_INTENSITY ]] && conf+=" --dual_tuner_weights=${TEAMREDMINER_INTENSITY}"

  [[ -n $TEAMREDMINER_USER_CONFIG2 ]] && conf+=" ${TEAMREDMINER_USER_CONFIG2}"

  conf+=" --${TEAMREDMINER_ALGO2}_end"

  echo "$conf" > "$MINER_CONFIG"
}
